from torchnlp.word_to_vector.char_n_gram import CharNGram
from torchnlp.word_to_vector.bpemb import BPEmb
from torchnlp.word_to_vector.fast_text import FastText
from torchnlp.word_to_vector.glove import GloVe
from torchnlp.word_to_vector.aliases import aliases

__all__ = ['GloVe', 'BPEmb', 'FastText', 'CharNGram', 'aliases']
